
/**
 * -----------------------------------------------------
 * MenuItem.installOn(li) adds menu item behavior to li.
 * A menu item appears in a menu and eventually has an
 * attached section that is displayed in a content pane
 * when the user clicks on the menu item.
 *
 * Logging of MenuItem.installOn() events can be turned
 * on by setting MenuItem.debug.installOn or
 * Behavior.debug.installOn to true.
 * Logging of menu item click events can be turned on by
 * setting MenuItem.debug.ONCLICK to true.
 */
new Behavior({
  name: "MenuItem",
  /**
   * MenuItem.installOn(li, parentMenu, memento)
   * installs menu item behavior on li.
   * The parentMenu argument should be the menu that
   * contains li.
   * The memento argument should be the object
   * that parentMenu will use to modify its menu target
   * when li is selected.
   */
  init: function(li, parentMenu, memento) {
    li.addEventListener("click", this.onclick, true);
    li.PARENT_MENU = parentMenu;
    li.MEMENTO = memento;
    li.SUBMENU = document.createElement("div");
    li.className = "unselected";
  },
  members: {
    ADD_SUBMENU: function(submenu) {
      this.SUBMENU.appendChild(submenu);
    },
    toString: function() {
      return "MenuItem<" + this.MEMENTO.name + ">";
    }
  }
});
/**
 * MenuItem.onclick is registered as an event listener
 * on all menu items.
 * It logs click events if debugging ONCLICK logging
 * is turned on.
 *
 * FIXME - can this just be assigned to the onclick member
 * of the menu item?
 */
MenuItem.onclick = function(event) {
  if (MenuItem.debug.ALL || MenuItem.debug.onclick) {
    var message = this.toString() + ".onclick()";
    Log.add(message);
  }
  try {
    this.PARENT_MENU.SELECT(this);
    event.stopPropagation();
    event.preventDefault();
  } catch(exception) {
    Log.showException(exception);
  }
  if (MenuItem.debug.ALL || MenuItem.debug.onclick) {
    Log.alert();
  }
};
MenuItem.create = function(a, parentMenu, memento) {
  var li = document.createElement("li");
  li.className = "link";
  li.appendChild(a);
  li.URL = new URL(a.href);
  this.installOn(li, parentMenu, memento);
  return li;
};
MenuItem.createDefault = function(a, parentMenu, memento) {
  if (a == null) {
    a = DOM.createElement("a", "Overview");
  }
  a.href = "";
  var menuItem = this.create(a, parentMenu, memento);
  return menuItem;
};

/**
 * -----------------------------------------------------
 * The MenuU educator educates DOM <div> elements with
 * class "menu" to be menus.
 * A menu contains menu items, at least one of which is
 * a default menu item: a menu item whose attached
 * section is the section that contains the menu.
 *
 * A menu is part of a cascaded menu.
 * As such it has
 */
new Behavior({
  name: "Menu",
  /**
   * Menu.installOn(element, section, frame) installs
   * menu behavior on element.
   * The argument section should be the section that
   * contains element.
   * The argument frame should be the display frame that
   * whose display is conntrolled by the menu.
   */
  init: function(element, section, frame) {
    element.SECTION = section;
    element.FRAME = frame;
    DOM.disconnect(element, "Menu.init");
    var title = element.title || frame.title;
    if (title) {
      var header = DOM.createElement("h4", title);
      DOM.insertFirst(element, header, "Menu.init");
    }
    var listElement = document.createElement("ul");
    DOM.appendChild(element, listElement, "Menu.init");
    var aList = [];
    var aTags = element.getElementsByTagName("a");
    for (var i = 0; i < aTags.length; i++) {
      aList.push(aTags.item(i));
    }
    var menuItems = [];
    for (var k = 0; k < aList.length; k++) {
      var a = aList[k];
      if (a.href) {
        var linkURL = new URL(a.href);
        var fragment = linkURL.fragment;
        var memento = {
          name: fragment,
          visual: Pane.createWrapper(fragment),
          url: linkURL
        };
        var menuItem = MenuItem.create(a, element, memento);
      } else {
        a.href = "";
        memento = {
          name: "@" + section.NAME,
          visual: section
        };
        menuItem = MenuItem.createDefault(a, element, memento);
        if (!firstDefault) {
          var firstDefault = menuItem;
        }
      }
      menuItems.push(menuItem);
    }
    if (!firstDefault) {
      memento = {
        name: "@" + section.NAME,
        visual: section
      };
      firstDefault = MenuItem.createDefault(null, element, memento);
      menuItems.unshift(firstDefault);
    }
    for (var l = 0; l < menuItems.length; l++) {
      menuItem = menuItems[l];
      DOM.appendChild(listElement, menuItem, "Menu.init");
    }
    element.SELECT(firstDefault);
  },
  members: {
    /**
     * Menu.SELECT is installed in all menus.
     * If menuItem is the current selected item of menu
     * then menu.SELECT(menuItem) does nothing.
     * If menuItem is not the current selected item of
     * menu then menu.SELECT(menuItem) deselects the
     * current selected item and makes menuItem the new
     * current selected item of menu.
     */
    SELECT: function(menuItem) {
      if (Menu.debug.ALL || Menu.debug.SELECT) {
        var message = this.toString() + ".SELECT(";
        message += menuItem.toString() + ")\n";
        Log.add(message);
      }
      var oldSelectedItem = this.SELECTED_ITEM;
      if (oldSelectedItem) {
        if (menuItem == oldSelectedItem) {
          return;
        }
        oldSelectedItem.className = "unselected";
        var child = oldSelectedItem.SUBMENU;
        DOM.removeChild(this, child, "Menu.SELECT");
      }
      menuItem.className = "selected"
      this.SELECTED_ITEM = menuItem;
      DOM.appendChild(this, menuItem.SUBMENU, "Menu.SELECT");
      var memento = menuItem.MEMENTO;
      if (memento.url) {
        var context = {
          url: memento.url,
          callback: function(context) {
            var section = context.element;
            var menuItem = context.targetMenuItem;
            var visual = menuItem.MEMENTO.visual;
            Section.installOn(section, visual);
            if (section.MENU) {
              menuItem.ADD_SUBMENU(section.MENU);
            }
          },
          targetMenuItem: menuItem
        };
        Repository.loadElement(context);
        delete memento.url;
      }
      this.FRAME.setVisual(memento.visual);
    },
    toString: function() {
      return "Menu<" + this.SECTION.NAME + ">";
    }
  }
});
