
/**
 * -----------------------------------------------------
 * The Page function is the constructor for objects of
 * class Page.
 * Each Page object represents a separately loaded web
 * page specified by the constructor url argument.
 * If the url argument is omitted or null then the page
 * represents the current document.
 */
function Page(url) {
  if (!url) {
    this.url = URL.getDocumentURL();
    this.status = 200;
    var elements = document.body.childNodes;
    var elementArray = [];
    for (var i = 0; i < elements.length; i++) {
      var element = elements.item(i);
      elementArray.push(element);
    }
    var pseudoDocument = [];
    for (var j = 0; j < elementArray.length; j++) {
      element = elementArray[j];
      document.body.removeChild(element);
      if (element.id) {
        pseudoDocument[element.id] = element;
      }
    }
    pseudoDocument.getElementById = function(id) {
      return this[id];
    }
    this.responseXML = pseudoDocument;
  } else {
    this.url = url;
    this.contexts = [];
    this.callback = function(thisPage) {
      thisPage.executeCallbacks();
    }
    Loader.loadHTTP(this);
  }
};
/**
 * intent: private
 * After handling load errors page.executeCallbacks()
 * executes the callbacks that have been passed in
 * parameters to page.loadElement() since page was
 * created.
 */
Page.prototype.executeCallbacks = function() {
  if (this.exception) {
    LoadErrorPage.installOn(this);
  } else if (!this.responseXML) {
    this.exception = "Loaded page is not valid XHTML.\n"
    this.exception += "Possible causes are: <ul>\n";
    this.exception += "<li>The file is an ordinary HTML file.</li>\n";
    this.exception += "<li>The file does not exist; ";
    this.exception += "an ordinary HTML error page was received.</li></ul>\n";
    LoadErrorPage.installOn(this);
  }
  for (var i in this.contexts) {
    var context = this.contexts[i];
    context.element = this.getElement(context.elementID);
    context.callback(context);
  }
  delete this.contexts;
};
/**
 * intent: private
 * page.getElement(elementID) returns the element in page
 * that has id elementID.
 * If there is no such element a load error element is
 * returned.
 */
Page.prototype.getElement = function(elementID) {
  var element = this.responseXML.getElementById(elementID);
  if (element != null) {
    var newElement = Importer.importElement(element, this.url);
    newElement.IDENTIFIER = newElement.id;
    newElement.URL = this.url.path + "#" + newElement.IDENTIFIER;
    delete newElement.id;
    return newElement;
  }
  var message = "Element with id #" + elementID + " not found.";
  element = DOM.createErrorElement("Element Load", this.url, message);
  return element;
};
/**
 * page.loadElement(context) adds an element imported from
 * page to context asynchronously.
 * The parameter context should initially be an object with
 * two attributes: elementID and callback:
 *   context.elementID specifies the id for the loaded element.
 *   context.callback is a function that will be invoked
 *     when the load operation has completed.
 *     It will be invoked with the context object as its
 *     only argument.
 *
 * After the load the context object will have one new
 * attribute:
 *   context.element - the loaded element.
 */
Page.prototype.loadElement = function(context) {
  if (!this.status) {
    this.contexts.push(context);
    return;
  }
  context.element = this.getElement(context.elementID);
  context.callback(context);
};
/**
 * page.toString() returns a string describing page.
 * The string has the form "Page<id>" where id is
 * the page id of the web page referenced by page.
 */
Page.prototype.toString = function() {
  return "Page<" + this.url.pageID + ">";
};

/**
 * LoadErrorPage.installOn(page) installs load error
 * page behavior on page.
 * When this behavior is installed the getElement()
 * method of page always ignores its context argument
 * and returns an error element that describes the page
 * load error.
 */
new Behavior({
  name: "LoadErrorPage",
  members: {
    /**
     * This method overrides Page.loadElement(context) to
     * ignore context.fragment and just add an error
     * element to context.
     */
    getElement: function(context) {
      return DOM.createErrorElement("Page Load", this.url, this.exception);
    }
  }
});

/**
 * -----------------------------------------------------
 * The Repository object provides access to elements
 * that make up a complex document.
 */
Repository = {
  debug: {},
  /**
   * pages is the list of Page objects held by
   * Repository.
   */
  pages: {},
  /**
   * Intent: private.
   * Repository.addPage(page) adds page to Repository.
   */
  addPage: function(page) {
    var pageID = page.url.pageID;
    this.pages[pageID] = page;
  },
  /**
   * Intent: private.
   * Repository.getPage(context) returns the page in
   * Repository whose url is context.url.
   * If Repository does not contain such a page a new
   * page is created.
   */
  getPage: function(context) {
    var url = context.url;
    var pageID = url.pageID;
    var page = this.pages[pageID]
    if (this.pages[pageID] == null) {
      page = new Page(url);
      this.addPage(page);
    }
    return page;
  },
  /**
   * Repository.initialize() creates a page for the
   * browser's current document.
   */
  initialize: function() {
    var documentPage = new Page();
    this.addPage(documentPage);
    return documentPage.getElement("ROOT");
  },
  /**
   * Repository.loadElement(context) adds an element from a
   * loaded document to context asynchronously.
   * The parameter context should initially be an object with
   * two attributes: url and callback:
   *   context.url specifies the url for the loaded document.
   *     This url should have a fragment part with the id of the
   *     desired element.
   *   context.callback is a function that will be invoked
   *     when the load operation has completed.
   *     It will be invoked with the context object as its
   *     only argument.
   *
   * After the load the context object will have one new
   * attribute:
   *   context.element - the loaded element.
   */
  loadElement: function(context) {
    context.elementID = context.url.fragment.substring(1);
    this.getPage(context).loadElement(context);
  }
};
