/**
 * The CSSClass object contains utility functions for working with the class
 * attribute (className property) of HTML elements.
 * All functions take two arguments: the element being tested or manipulated
 * and the CSS class that is to be tested, added, or removed.
 *
 * This code is a modified copy of Example 16-7 from the book JavaScript:
 * The Definitive Guide, 5th Edition, by David Flanagan. Copyright 2006
 * O'Reilly Media, Inc.
 * ISBN: 0596101996
 */
var CSSClass = {
  /**
   * CSSClass.hasClass(element, cname) returns true if element is a member of
   * the class cname and false if not.
   */
  hasClass: function(e, c) {
    // Before doing a regexp search, optimize for a couple of common cases.
    var classes = e.className;
    if (!classes) {
      return false;
    }
    if (classes == c) {
      return true;
    }
    // Otherwise, use a regular expression to search for c as a word by
    // itself.
    // \b in a regular expression requires a match at a word boundary.
    return e.className.search("\\b" + c + "\\b") != -1;
  },
  /**
   * CSSClass.add(element, cname) adds class cname to the className of element
   * if it is not already there.
   */
  add: function(e, c) {
    if (CSSClass.isClass(e, c)) {
      return;
    }
    if (e.className) {
      c = " " + c;
    }
    e.className += c;
  },
  /**
   * CSSClass.remove(element, cname) removes all occurrences (if any) of class
   * cname from the className of element.
   */
  remove: function(e, c) {
    // Search the className for all occurrences of c and replace with "".
    // \s* matches any number of whitespace characters.
    // \b matches a word boundary
    // "g" makes the regular expression match any number of occurrences.
    var testExpression = new RegExp("\\b" + c + "\\b\\s*", "g");
    e.className = e.className.replace(testExpression, "");
  }
}
