
/** MenuItemU
 * The MenuItemU educator educates DOM <li> elements
 * with class "link" to be menu items.
 * It also directs further education of menu items.
 * A menu item appears in a menu and eventually has an
 * attached section that is displayed in a content pane
 * when the user clicks on the menu item.
 *
 * Logging of student graduation events for menu items
 * can be turned on by setting MenuItemU.debug.GRADUATION
 * or Education.debug.GRADUATION to true.
 * Logging of menu item click events can be turned on by
 * setting MenuItemU.debug.ONCLICK to true.
 * Logging of menu item display activity can be turned
 * on by setting MenuItemU.debug.SHOW to true.
 * Logging of menu item section attachment can be turned
 * on by setting MenuItemU.debug.ATTACH_SECTION to true.
 * Logging of click events, display activity, and
 * section attachment can all be turned on by setting
 * MenuItemU.debug.ALL to true.
 */
MenuItemU = {
  // Teacher members
  debug: {},
  name: "MenuItem",
  conferDegree: Education.conferDegree,
  /** MenuItemU.teach
   * MenuItemU.teach(@li, @parentMenu, @section) educates
   * @li to be a menu item and directs the further
   * education of @li to be a default menu item, error
   * menu item, or deferred menu item.
   * @parentMenu should be the menu that contains li.
   * @section should be the section that contains li.
   */
  teach: function(li, parentMenu, section) {
    // Education setup
    li.IDENTIFIER = li.toString();
    li.toString = Education.toString;
    // Add student methods
    li.DO_SHOW = this.DO_SHOW;
    li.SHOW = this.SHOW;
    li.addEventListener("click", this.onclick, true);
    li.DO_ONCLICK = this.DO_ONCLICK;
    // Process li
    li.PARENT_MENU = parentMenu;
    li.className = "unselected";
    // Graduate li as a menu item
    this.conferDegree(li);
    // determine future education of li
    var anchorList = li.getElementsByTagName("a");
    if (anchorList.length > 0) {
      var anchor = anchorList[0];
      if (anchor.href) {
        li.URL = new URL(anchor.href);
        li.IDENTIFIER = li.URL.fragment;
        DeferredMenuItemU.teach(li);
      } else {
        anchor.href = "";
        DefaultMenuItemU.teach(li, section);
      }
    } else {
      ErrorMenuItemU.teach(li, "unknown url",
        "Link has no &lt;a&gt; tag");
    }
  },
  /** MenuItemU.onclick
   * MenuItem.onclick is installed in all menu items.
   * It logs click events if debugging ONCLICK logging
   * is turned on, then invokes DO_ONCLICK().
   * This method should not be replaced by other menu
   * item educators.
   * They should replace DO_ONCLICK.
   */
  onclick: function(event) {
    if (MenuItemU.debug.ALL || MenuItemU.debug.ONCLICK) {
      var message = this.toString() + ".onclick()";
      Log.add(message);
    }
    try {
      this.PARENT_MENU.SELECT(this);
      this.DO_ONCLICK(event);
      event.stopPropagation();
      event.preventDefault();
    } catch(exception) {
      Log.showException(exception);
    }
  },
  /** MenuItemU.DO_ONCLICK
   * MenuItem.DO_ONCLICK is installed in all menu items.
   * This method does not do anything.
   * Other menu item educators can replace the
   * MenuItemU DO_ONCLICK method with their own
   * DO_ONCLICK methods.
   */
  DO_ONCLICK: function(event) {
  },
  /** MenuItemU.SHOW
   * MenuItem.SHOW is installed in all menu items.
   * It logs click display activity if debugging SHOW
   * logging is turned on, then invokes DO_SHOW().
   * This method should not be replaced by other menu
   * item educators.
   * They should replace DO_SHOW.
   */
  SHOW: function(pane) {
    if (MenuItemU.debug.ALL || MenuItemU.debug.SHOW) {
      var message = this.toString() + ".SHOW()";
      message += pane.toString() + ")";
      Log.add(message);
    }
    this.DO_SHOW(pane);
  },
  /** MenuItemU.DO_SHOW
   * MenuItem.DO_SHOW is installed in all menu items.
   * This method does not do anything.
   * Other menu item educators can replace the
   * MenuItemU DO_SHOW method with their own DO_SHOW
   * methods once the menu item has a section to be
   * displayed in the content pane.
   */
  DO_SHOW: function(pane) {
  }
};

/** ErrorMenuItemU
 * ErrorMenuItemU educates menu items to be error menu
 * items.
 * An error menu item represents an error on the part of
 * the presentation author in the specification of a
 * link.
 * It is a leaf menu item whose section describes the
 * nature of the error.
 */
ErrorMenuItemU = {
  // Teacher members
  debug: {},
  name: "ErrorMenuItem",
  conferDegree: Education.conferDegree,
  /** ErrorMenuItemU.teach
   * ErrorMenuItemU.teach(@menuItem, @url, @description)
   * teaches @menuItem to be an error menu item.
   * @url should be the string url for the error, or
   * just "unknown url" if the url is not known.
   * @description should be a string describing the
   * nature of the error.
   */
  teach: function(menuItem, url, description) {
    // Process menuItem
    var section = DOM.createErrorElement("Link", url,
      description);
    SectionU.teach(section);
    menuItem.className = "link-error";
    menuItem.IDENTIFIER = "ERROR";
    LeafMenuItemU.teach(menuItem, section);
    // Graduate menuItem
    this.conferDegree(menuItem);
  }
};

/** DeferredMenuItemU
 * DeferredMenuItemU educates a menu item to be a
 * deferred menu item.
 * A deferred menu item does not yet have a section
 * attached to it.
 * Its section will be attached when the menu item is
 * clicked.
 * At this time it will become either a leaf menu item
 * or a nonleaf menu item depending on whether or not
 * the section contains a menu.
 *
 * Logging of menu item section attachment can be turned
 * on by setting MenuItemU.debug.ATTACH_SECTION to true.
 */
DeferredMenuItemU = {
  // Teacher members
  debug: {},
  name: "DeferredMenuItem",
  conferDegree: Education.conferDegree,
  /** DeferredMenuItemU.teach
   * DeferredMenuItemU.teach(@menuItem) teaches @menuItem
   * to be a deferred menu item.
   */
  teach: function(menuItem) {
    // Replace student methods
    menuItem.DO_ONCLICK = this.DO_ONCLICK;
    // Graduate menuItem
    this.conferDegree(menuItem);
  },
  /** PRIVATE DeferredMenuItemU.attachSection
   * DeferredMenuItemU.attachSection is the callback
   * that students should add to their load context
   * passed to Repository.loadElement().
   * It educates the returned element to be a section
   * and directs further education of the deferred
   * menu item to be either a leaf menu item or a
   * nonleaf menu item, depending on whether or not
   * the section contains a menu.
   */
  attachSection: function(context) {
    var section = context.element;
    var menuItem = context.menuItem;
    if (MenuItemU.debug.ALL || MenuItemU.debug.ATTACH_SECTION) {
      var message = "MenuItemU.ATTACH_SECTION(";
      message += menuItem.toString() + ", ";
      message += section.toString() + ")";
      Log.add(message);
    }
    SectionU.teach(section);
    menuItem.removeEventListener("click",
      DeferredMenuItemU.onclick, true);
    if (section.MENU) {
      NonleafMenuItemU.teach(menuItem, section);
    } else {
      LeafMenuItemU.teach(menuItem, section);
    }
    menuItem.PARENT_MENU.ACTIVATE_CHILD(menuItem);
  },
  /** DeferredMenuItemU.DO_ONCLICK
   * DeferredMenuItemU.DO_ONCLICK is installed in all
   * deferred menu items.
   * @menuItem.DO_ONCLICK(@event) asynchronously attaches
   * a section to @menuItem.
   */
  DO_ONCLICK: function(event) {
    var loadContext = {
      url: this.URL,
      menuItem: this,
      callback: DeferredMenuItemU.attachSection
    };
    Repository.loadElement(loadContext);
  }
};

/** LeafMenuItemU
 * LeafMenuItemU educates menu items to be leaf menu
 * items.
 * A leaf menu item has a section attached to it.
 * The section does not contain a menu.
 */
LeafMenuItemU = {
  // Teacher members
  debug: {},
  name: "LeafMenuItem",
  conferDegree: Education.conferDegree,
  /** LeafMenuItemU.teach
   * LeafMenuItemU.teach(@menuItem, @section) teaches
   * @menuItem to be a leaf menu item.
   * @section should be the section attached to menuItem.
   * It should not contain a menu.
   */
  teach: function(menuItem, section) {
    // Replace student methods
    menuItem.DO_SHOW = this.DO_SHOW;
    menuItem.DO_ONCLICK = MenuItemU.DO_ONCLICK;
    // Process menuItem
    menuItem.SECTION = section;
    // Graduate menuItem
    this.conferDegree(menuItem);
  },
  // Methods added to students
  /**
   * LeafMenuItemU.DO_SHOW is installed in all leaf
   * menu items.
   * @menuItem.DO_SHOW(@pane) sets the visual for @pane
   * to the section attached to @menuItem.
   */
  DO_SHOW: function(pane) {
    Layout.setVisual(pane, this.SECTION);
  }
};

/** DefaultMenuItemU
 * DefaultMenuItemU educates menu items to be default
 * menu items.
 * A default menu item has a section attached to it.
 * The section is the section that contains the parent
 * menu for the menu item.
 */
DefaultMenuItemU = {
  // Teacher members
  debug: {},
  name: "DefaultMenuItem",
  conferDegree: Education.conferDegree,
  /** DefaultMenuItemU.teach
   * DefaultMenuItemU.teach(@menuItem, @section) teaches
   * @menuItem to be a default menu item for @section.
   */
  teach: function(menuItem, section) {
    // Process menuItem
    menuItem.IDENTIFIER = section.toString();
    LeafMenuItemU.teach(menuItem, section);
    // Graduate menuItem
    this.conferDegree(menuItem);
  },
  /** DefaultMenuItemU.create
   * DefaultMenuItemU.create(@section) creates a default
   * menu item for @section.
   * The new default menu item has the label "Overview".
   */
  create: function(section) {
    var a = DOM.createElement("a", "Overview");
    a.href = "";
    var li = document.createElement("li");
    li.className = "link";
    li.appendChild(a);
    MenuItemU.teach(li, section.MENU);
    DefaultMenuItemU.teach(li, section);
    return li;
  }
};

/** NonleafMenuItemU
 * NonleafMenuItemU educates menu items to be nonleaf
 * menu items.
 * A nonleaf menu item has a section attached to it.
 * The section contains a menu.
 */
NonleafMenuItemU = {
  // Teacher members
  debug: {},
  name: "NonleafMenuItem",
  conferDegree: Education.conferDegree,
  /** NonleafMenuItemU.teach
   * NonleafMenuItemU.teach(@menuItem, @section) teaches
   * @menuItem to be a nonleaf menu item.
   * @section should be the section attached to menuItem.
   * It should contain a menu.
   */
  teach: function(menuItem, section) {
    // Add student methods
    menuItem.DO_SHOW = this.DO_SHOW;
    menuItem.DO_ONCLICK = MenuItemU.DO_ONCLICK;
    // Process div
    menuItem.SUBMENU = section.MENU;
    menuItem.SUBMENU.SELECTOR = menuItem;
    // Graduate menuItem
    this.conferDegree(menuItem);
  },
  // Methods added to students
  /** NonleafMenuItemU.DO_SHOW
   * NonleafMenuItemU.DO_SHOW is installed in all
   * nonleaf menu items.
   * @menuItem.DO_SHOW(@pane) forwards the SHOW(@pane)
   * message to the selected menu item of @menuItem's
   * submenu.
   */
  DO_SHOW: function(pane) {
    this.SUBMENU.SELECTED_ITEM.SHOW(pane);
  }
};

/** MenuU
 * MenuU educates DOM <div> elements with class "menu"
 * to be menus.
 * A menu contains menu items, at least one of which is
 * a default menu item: a menu item whose attached
 * section is the section that contains the menu.
 */
MenuU = {
  // Teacher members
  debug: {},
  name: "Menu",
  conferDegree: Education.conferDegree,
  /** MenuU.teach
   * MenuU.teach(@element, @section) teaches @element
   * to be a menu.
   * @section should be the section that contains
   * @element.
   */
  teach: function(element, section) {
    // Education setup
    element.IDENTIFIER = section.IDENTIFIER;
    element.DEGREE = "PreMenu";
    element.toString = Education.toString;
    // Add student methods
    element.SELECT = this.SELECT;
    element.ACTIVATE_CHILD = this.ACTIVATE_CHILD;
    element.IS_SELECTED = this.IS_SELECTED;
    // Process div
    element.SECTION = section;
    DOM.disconnect(element, "MenuU.teach");
    var liList = element.getElementsByTagName("li");
    var firstDefault = null;
    for (var j = 0; j < liList.length; j++) {
      var link = liList.item(j);
      MenuItemU.teach(link, element, section);
      if (link.DEGREE == "DefaultMenuItem" && firstDefault == null) {
        firstDefault = link;
      }
    }
    if (firstDefault == null) {
      firstDefault = MenuU.addDefaultMenuItem(element, section);
    }
    element.SELECTED_ITEM = firstDefault;
    firstDefault.className = "selected";
    // Graduate menu
    this.conferDegree(element);
  },
  /** PRIVATE MenuItemU.addDefaultMenuItem
   * MenuItemU.addDefaultMenuItem(@menu, @section) adds a
   * default menu item to @menu.
   * @section is attached to the default menu item.
   */
  addDefaultMenuItem: function(menu, section) {
    var defaultMenuItem = DefaultMenuItemU.create(section);
    var itemParent = null;
    var listList = menu.getElementsByTagName("ul");
    if (listList.length > 0) {
      itemParent = listList[0];
    }
    if (itemParent == null) {
      listList = menu.getElementsByTagName("ol");
      if (listList.length > 0) {
        itemParent = listList[0];
      } else {
        itemParent = menu;
      }
    }
    DOM.insertFirst(itemParent, defaultMenuItem,
      "addDefaultMenuItem");
    return defaultMenuItem;
  },
  // Methods added to students
  /** MenuU.SELECT
   * MenuU.SELECT is installed in all menus.
   * If @menuItem is the current selected item of @menu
   * then @menu.SELECT(@menuItem) does nothing.
   * If @menuItem is not the current selected item of
   * @menu then @menu.SELECT(@menuItem) deselects the
   * current selected item and makes @menuItem the new
   * current selected item of @menu.
   */
  SELECT: function(menuItem) {
    if (MenuU.debug.ALL || MenuU.debug.SELECT) {
      var message = this.toString() + ".SELECT(";
      message += menuItem.toString() + ")\n";
      Log.add(message);
    }
    var oldSelectedItem = this.SELECTED_ITEM;
    if (menuItem == oldSelectedItem) {
      return;
    }
    if (oldSelectedItem.IDENTIFIER == "ERROR") {
      oldSelectedItem.className = "link-error";
    } else {
      oldSelectedItem.className = "unselected";
    }
    if (oldSelectedItem.SUBMENU) {
      var child = oldSelectedItem.SUBMENU;
      DOM.removeChild(this, child, "MenuU.SELECT");
    }
    menuItem.className = "selected"
    this.SELECTED_ITEM = menuItem;
    if (menuItem.SUBMENU) {
      DOM.appendChild(this, menuItem.SUBMENU, "MenuU.SELECT");
    }
    menuItem.SHOW(Layout.currentPane);
  },
  /** MenuU.ACTIVATE_CHILD
   * MenuU.ACTIVATE_CHILD is installed in all menus.
   * @menu.ACTIVATE_CHILD(@menuItem) should only be invoked
   * when @menuItem gets a section attached to it.
   * If @menuItem is still selected by @menu and it has a
   * submenu then its submenu will be appended to @menu.
   * If @menu also appears in the current menu cascade
   * then @menuItem's section will be shown in the content
   * pane.
   */
  ACTIVATE_CHILD: function(menuItem) {
    if (MenuU.debug.ALL || MenuU.debug.ACTIVATE_CHILD) {
      var message = this.toString() + ".ACTIVATE_CHILD(";
      message += menuItem.toString() + ")\n";
      Log.add(message);
    }
    if (menuItem != this.SELECTED_ITEM) {
      return;
    }
    if (menuItem.SUBMENU) {
      DOM.appendChild(this, menuItem.SUBMENU, "MenuU.SELECT");
    }
    if (this.IS_SELECTED()) {
      menuItem.SHOW(Layout.currentPane);
    }
  },
  /** MenuU.IS_SELECTED
   * MenuU.IS_SELECTED is installed in all menus.
   * @menu.IS_SELECTED() returns true if the menu item
   * to which the section containing @menu is attached
   * is currently selected.
   * This happens when and only when @menu appears in the
   * menu cascade.
   */
  IS_SELECTED: function() {
    if (MenuU.debug.ALL || MenuU.debug.IS_SELECTED) {
      var message = this.toString() + ".IS_SELECTED()\n";
      Log.add(message);
    }
    if (!this.SELECTOR) {
      return true;
    }
    if (this.SELECTOR.className != "selected") {
      return false;
    }
    return this.SELECTOR.PARENT_MENU.IS_SELECTED();
  }
};
