
/** Page
 * The Page function is the constructor for page
 * objects.
 * A page object represents a web page.
 * By invoking its loadElement() method, it provides
 * access to elements in the web page that have their
 * id attribute set.
 */
/** Page.new
 * new Page(@url) returns a new page object for the web
 * page whose url is @url.
 * If @url is not a valid url or if the web page it
 * refers to does not exist then a load error page is
 * returned.
 */
function Page(url) {
  if (!url) {
    this.url = URL.getDocumentURL();
    this.responseXML = document;
    this.status = 200;
    var elements = document.body.childNodes;
    var elementArray = [];
    for (var i = 0; i < elements.length; i++) {
      var element = elements.item(i);
      elementArray.push(element);
    }
    var bodyCopy = document.createElement("div");
    bodyCopy.style.display = "none";
    for (var j = 0; j < elements.length; j++) {
      element = elementArray[j];
      bodyCopy.appendChild(element);
    }
    document.body.appendChild(bodyCopy);
  } else {
    this.url = url;
    this.contexts = [];
    this.callback = function(thisPage) {
      thisPage.executeCallbacks();
    }
    Loader.loadHTTP(this);
  }
};
/** PRIVATE Page.executeCallbacks
 * @page.executeCallbacks() executes the callbacks that
 * have been passed in as arguments to @page.loadElement()
 * since @page was created.
 */
Page.prototype.executeCallbacks = function() {
  if (this.exception) {
    LoadErrorPageU.teach(this);
  } else if (!this.responseXML) {
    this.exception = "Loaded page is not valid XHTML.\n"
    this.exception += "Possible causes are: <ul>\n";
    this.exception += "<li>The file is an ordinary HTML file.</li>\n";
    this.exception += "<li>The file does not exist; ";
    this.exception += "an ordinary HTML error page was received.</li></ul>\n";
    LoadErrorPageU.teach(this);
  }
  for (var i in this.contexts) {
    var context = this.contexts[i];
    context.element = this.getElement(context.elementID);
    context.callback(context);
  }
  delete this.contexts;
};
/** PRIVATE Page.getElement
 * @page.getElement(@elementID) returns the element in page
 * that has id @elementID.
 * If there is no such element a load error element is
 * returned.
 */
Page.prototype.getElement = function(elementID) {
  var element = this.responseXML.getElementById(elementID);
  if (element != null) {
    var newElement = Importer.importElement(element, this.url);
    newElement.IDENTIFIER = newElement.id;
    delete newElement.id;
    return newElement;
  }
  var message = "Element with id #" + elementID + " not found.";
  element = DOM.createErrorElement("Element Load",
    this.url, message);
  return element;
};
/** Page.loadElement
 * @page.loadElement(@context) adds an element imported from
 * @page to @context asynchronously.
 * @context should initially be an object with two attributes:
 * elementID and callback:
 *
 *   @context.elementID - specifies the id for the loaded element.
 *   
 *   @context.callback - a function that will be invoked
 *     when the load operation has completed.
 *     It will be invoked with the context object as its
 *     only argument.
 *
 * After the load @context will have one new attribute:
 * 
 *   @context.element - the loaded element.
 */
Page.prototype.loadElement = function(context) {
  if (!this.status) {
    this.contexts.push(context);
    return;
  }
  context.element = this.getElement(context.elementID);
  context.callback(context);
};
/** Page.toString
 * @page.toString() returns a string describing page.
 * The string has the form "Page<id>" where id is
 * the page id of the web page referenced by @page.
 */
Page.prototype.toString = function() {
  return "Page<" + this.url.pageID + ">";
};

/** LoadErrorPageU
 * The LoadErrorPageU educator object teaches pages to be
 * load error pages.
 * A load error page has only one element which describes
 * the error that occured when the page attempted to access
 * its web page.
 */
LoadErrorPageU = {
  /** LoadErrorPageU.teach
   * LoadErrorPageU.teach(@page) teaches @page to be a load
   * error page.
   */
  teach: function(page) {
    page.getElement = LoadErrorPageU.getElement;
  },
  /** PRIVATE LoadErrorPageU.getElement
   * This method is installed in all load error pages
   * overriding PageU.getElement.
   * @page.loadElement(@context) ignores @context.fragment
   * and just adds a load error element to @context.
   */
  getElement: function(context) {
    return DOM.createErrorElement("Page Load",
      this.url, this.exception);
  }
};

/** Repository
 * The Repository object provides access to elements
 * that make up a complex document.
 */
Repository = {
  debug: {},
  /** PRIVATE Repository.pages
   * Repository.pages is the list of page objects held
   * by Repository.
   */
  pages: {},
  /** PRIVATE Repository.addPage
   * Repository.addPage(@page) adds @page to Repository.
   */
  addPage: function(page) {
    var pageID = page.url.pageID;
    this.pages[pageID] = page;
  },
  /** PRIVATE Repository.getPage
   * Repository.getPage(@context) returns the page in
   * Repository whose url is @context.url.
   * If Repository does not contain such a page a new
   * page is created.
   */
  getPage: function(context) {
    var url = context.url;
    var pageID = url.pageID;
    var page = this.pages[pageID]
    if (this.pages[pageID] == null) {
      page = new Page(url);
      this.addPage(page);
    }
    return page;
  },
  /** Repository.initialize
   * Repository.initialize() creates a page for the
   * browser's current document.
   */
  initialize: function() {
    var documentPage = new Page();
    this.addPage(documentPage);
    return documentPage.getElement("ROOT");
  },
  /** Repository.loadElement
   * Repository.loadElement(@context) adds an element from
   * a loaded document to @context asynchronously.
   * The argument @context should initially be an object
   * with two attributes: url and callback:
   *
   *   @context.url -
   *     specifies the url for the loaded document.
   *     The fragment part of @context.url should be the id
   *     of the desired element.
   *
   *   @context.callback -
   *     a function that will be invoked when the load
   *     operation has completed.
   *     It will be invoked with @context as its only
   *     argument.
   *
   * After the load @context will have one new attribute:
   *   @context.element -
   *     the loaded element.
   */
  loadElement: function(context) {
    context.elementID = context.url.fragment.substring(1);
    this.getPage(context).loadElement(context);
  }
};
