
/**
 * -----------------------------------------------------
 * The Loader object provides asynchronous file load
 * capabilities through its loadHTTP method.
 */
Loader = {
  /**
   * Loader.loadHTTP(context) attaches a loaded document
   * to context asynchronously.
   * The parameter context should initially be an object with
   * two or three attributes: url, callback, and timeout:
   *   context.url specifies the url for the loaded document.
   *   context.callback is a function that will be invoked
   *     when the load operation has completed.
   *     It will be invoked with the context object as its
   *     only argument.
   *   context.timeout specifies the timeout in milliseconds.
   *     If the document fails to load before its timeout
   *     the the load is treated as an exception.  If this
   *     attribute is not defined a default timeout of 5000
   *     (5 seconds) will be assigned to it.
   *
   * After the load the context object will have three new
   * attributes:
   *   context.status - the status for the load.
   *   context.responseText - the loaded text.
   *   context.responseXML - an XML document parsed from
   *     the load.
   * The responseXML attribute is null if the loaded document
   * is not valid XML or XHTML.
   *
   * The possible values for context.status are:
   *   200 - successful load.
   *   400 - exception thrown while loading.
   *   404 - file not found.
   * If context.status is not 200 then context.exception is
   * a description of the exception.
   */
  loadHTTP: function(context) {
    var request = new XMLHttpRequest();
    var timer;
    request.onreadystatechange = function() {
      if (request.readyState == 4) {
        clearTimeout(timer);
        if (request.status == 0 || request.status == 200) {
          context.status = 200;
          context.responseText = request.responseText;
          context.responseXML = request.responseXML;
        } else {
          context.status = 404;
        }
        context.callback(context);
      }
    }
    var timeout = context.timeout ? context.timeout : 5000;
    var timeoutCallback = function() {
      request.abort();
      context.status = 400;
      context.exception = "Load timed out after " +
      context.timeout/1000.0 + " seconds.";
      context.callback(context);
    }
    try {
      timer = setTimeout(timeoutCallback, timeout);
      request.open("GET", context.url);
      request.send(null);
    } catch(ex) {
      clearTimeout(timer);
      context.status = 400;
      context.exception = ex;
      context.callback(context);
    }
  }
};

/**
 * -----------------------------------------------------
 * The Log singleton object accepts messages from
 * other JavaScript components when its add()
 * method is invoked.
 * At a later time, all accumulated messages can
 * be displayed in an alert by invoking its alert()
 * method.
 */
Log = {
  messages: [],
  /**
   * Log.add(message) adds message to Log's list of
   * messages.
   */
  add: function(message) {
    if (!this.enabled) {
      return;
    }
    this.messages.push(message);
  },
  /**
   * Log.showException(exception) adds a message from
   * exception to Log's list of messages and then
   * invokes Log.alert().
   * On a Firefox browser the added message will show
   * a stack trace.
   */
  showException: function(exception) {
    if (!this.enabled) {
      throw exception;
      return;
    }
    var message = "Exception thrown: " + exception.message;
    if (exception.stack) {
      message += "\nStack trace follows:\n";
      message += exception.stack;
    } else {
      message += "\nNo stack trace"
    }
    this.messages.push(message);
    this.alert();
  },
  /**
   * Log.alert() pops up an alert that displays all
   * messages that have been added to Log since the
   * previous time its alert() method was invoked.
   * After the alert is dismissed all messages are
   * removed from Log.
   *
   * Each message has a newline character appended
   * automatically.
   */
  alert: function() {
    if (!this.enabled || this.messages.length == 0) {
      return;
    }
    var alertText = "";
    for (var i = 0; i < this.messages.length; i++) {
      alertText += this.messages[i] + "\n";
    }
    alertText += "\n";
    alert(alertText);
    this.messages.length = 0;
  },
  /**
   * Log.getCount() the number of messages that have
   * been added to Log since the previous time its
   * alert() method was invoked.
   */
  getCount: function() {
    return this.messages.length;
  }
};

/**
 * -----------------------------------------------------
 * The DOM object provides an assortment of services for
 * DOM element creation and modification.
 * Five of its methods support logging of their invocations.
 * Logging for these methods can be turned on by setting the
 * following variables to true.
 *
 * removeChild():  DOM.debug.REMOVE
 * appendChild():  DOM.debug.APPEND
 * insertFirst():  DOM.debug.INSERT
 * replaceChild(): DOM.debug.REPLACE
 * disconnect():   DOM.debug.DISCONNECT
 *
 * Logging for all five of these methods can be turned on by
 * setting DOM.debug.ALL to true.
 */
DOM = {
  debug: {},
  /**
   * DOM.createElement(type, text)
   * creates a DOM HTMLElement with tag type containing
   * a TextNode with content text.
   */
  createElement: function(type, text) {
    var element = document.createElement(type);
    element.appendChild(document.createTextNode(text));
    return element;
  },
  /**
   * DOM.createCompoundElement(text)
   * creates a DOM HTMLDIVElement containing text as its
   * innerHTML.
   */
  createCompoundElement: function(text) {
    var element = document.createElement("div");
    element.innerHTML = text;
    return element;
  },
  /**
   * DOM.createTableRow(object, name)
   * creates a DOM HTMLTableRowElement with two columns.
   * The first column displays name.
   * The second column displays object[name].
   */
  createTableRow: function(object, name) {
    var row = document.createElement("tr");
    var header = DOM.createElement("th", name);
    header.className = "header";
    row.appendChild(header);
    var value = DOM.createElement("td", object[name]);
    value.className = "value";
    row.appendChild(value);
    return row;
  },
  /**
   * DOM.substitute(element, valueTable)
   * substitutes values from valueTable into variable
   * tags in descendents of element.
   * A variable tag is a <span> tag with HTML class
   * "variable".
   * The name of the variable that it represents is
   * given by the "title" attribute of the tag.
   */
  substitute: function(element, values) {
    var spanTags = element.getElementsByTagName("span");
    for (var i = 0; i < spanTags.length; i++) {
      var tag = spanTags.item(i);
      if (tag.className == "variable") {
        var textNode = document.createTextNode(values[tag.title]);
        tag.appendChild(textNode);
      }
    }
  },
  /**
   * DOM.createErrorElement(type, url, details) returns an
   * element that indicates an error.
   * The parameter type should specify the type of the error.
   * The parameter url should specify the url of the page from
   * which the load was attempted.
   * The parameter details should contain html text describing
   * any further details about the error.
   */
  createErrorElement: function(type, url, details) {
    var element = document.createElement("div");
    element.URL = url;
    element.IDENTIFIER = url;
    element.appendChild(DOM.createElement("h2", type + " Error"));
    element.appendChild(DOM.createElement("h4", "Page URL:"));
    element.appendChild(DOM.createElement("p", url.toString()));
    element.appendChild(DOM.createElement("h4", "Error Details"));
    element.appendChild(DOM.createCompoundElement(details));
    element.className = "error";
    return element;
  },
  /**
   * DOM.removeChild(parent, child, where)
   * removes child from parent.
   * The where parameter is for debugging, describing
   * where this method is being called from.
   */
  removeChild: function(parent, child, where) {
    if (DOM.debug.ALL || DOM.debug.REMOVE) {
      var message = "In " + where + " removeChild:\n";
      message += "  parent = " + parent.toString() + ",\n";
      message += "  child = " + child.toString() + ",";
      Log.add(message);
    }
    try {
      parent.removeChild(child);
    } catch(exception) {
      try {
        throw new Error(exception.message);
      } catch(exception) {
        Log.showException(exception);
      }
    }
    if (DOM.debug.ALL || DOM.debug.REMOVE) {
      Log.add("  removeChild done.");
    }
  },
  /**
   * DOM.appendChild(parent, child, where)
   * inserts child into parent as its last child.
   * The where parameter is for debugging, describing
   * where this method is being called from.
   */
  appendChild: function(parent, child, where) {
    if (DOM.debug.ALL || DOM.debug.APPEND) {
      var message = "In " + where + " appendChild:\n";
      message += "  parent = " + parent.toString() + ",\n";
      message += "  child = " + child.toString() + ",";
      Log.add(message);
    }
    try {
      parent.appendChild(child);
    } catch(exception) {
      try {
        throw new Error(exception.message);
      } catch(exception) {
        Log.showException(exception);
      }
    }
    if (DOM.debug.ALL || DOM.debug.APPEND) {
      Log.add("  appendChild done.");
    }
  },
  /**
   * DOM.insertFirst(parent, child, where)
   * inserts child into parent as its first child.
   * The where parameter is for debugging, describing
   * where this method is being called from.
   */
  insertFirst: function(parent, child, where) {
    if (DOM.debug.ALL || DOM.debug.INSERT) {
      var message = "In " + where + " insertFirst:\n";
      message += "  parent = " + parent.toString() + ",\n";
      message += "  child = " + child.toString() + ",";
      Log.add(message);
    }
    try {
      parent.insertBefore(child, parent.firstChild);
    } catch(exception) {
      try {
        throw new Error(exception.message);
      } catch(exception) {
        Log.showException(exception);
      }
    }
    if (DOM.debug.ALL || DOM.debug.INSERT) {
      Log.add("  insertFirst done.");
    }
  },
  /**
   * DOM.replaceChild(parent, newChild, oldChild, where)
   * replaces oldChild with newChild in parent.
   * The where parameter is for debugging, describing
   * where this method is being called from.
   */
  replaceChild: function(parent, newChild, oldChild, where) {
    if (DOM.debug.ALL || DOM.debug.REPLACE) {
      var message = "In " + where + " replaceChild:\n";
      message += "  parent = " + parent.toString() + ",\n";
      message += "  new child = " + newChild.toString() + ",\n";
      message += "  old child = " + oldChild.toString() + ",";
      Log.add(message);
    }
    try {
      parent.replaceChild(newChild, oldChild);
    } catch(exception) {
      try {
        throw new Error(exception.message);
      } catch(exception) {
        Log.showException(exception);
      }
    }
    if (DOM.debug.ALL || DOM.debug.REPLACE) {
      Log.add("  replace done.");
    }
  },
  /**
   * DOM.disconnect(child, where)
   * removes child from its parent.
   * The where parameter is for debugging, describing
   * where this method is being called from.
   */
  disconnect: function(child, where) {
    if (DOM.debug.ALL || DOM.debug.DISCONNECT) {
      if (child.parentNode) {
        var parentString = child.parentNode.toString();
      } else {
        parentString = "undefined";
      }
      var message = "In " + where + " disconnect:\n";
      message += "  parent = " + parentString + ",\n";
      message += "  child = " + child.toString() + ",";
      Log.add(message);
    }
    if (child.ParentNode) {
      try {
        child.parentNode.removeChild(child);
      } catch(exception) {
        try {
          throw new Error(exception.message);
        } catch(exception) {
          Log.showException(exception);
        }
      }
    }
    if (DOM.debug.ALL || DOM.debug.DISCONNECT) {
      Log.add("  disconnect done.");
    }
  }
};

function Behavior(args) {
  this.debug = {};
  this.name = args.name;
  this.init = args.init;
  this.members = args.members;
  window[args.name] = this;
}
Behavior.debug = {};
Behavior.prototype.installOn = function(target) {
  if (this.members) {
    for (var name in this.members) {
      target[name] = this.members[name];
    }
  }
  if (this.init) {
    this.init.apply(this, arguments);
  }
  if (Behavior.debug.installOn || this.debug.installOn) {
    Log.add(this.name + " behavior installed on " + target.toString() + ".");
  }
}

new Behavior({
  name: "Named",
  init: function(target, name) {
    target.NAME = name;
  },
  members: {
    toString: function() {
      return this.NAME;
    }
  }
});
